git add -A
git commit --no-verify -m "Prepare template using blakeNaccarato/copier-python"
git submodule deinit --all
git submodule add --force --name template https://github.com/blakeNaccarato/copier-python.git submodules/template
git submodule add --force --name typings https://github.com/blakeNaccarato/pylance-stubs-unofficial.git submodules/typings
git add -A
git commit --no-verify -m "Add template and type stub submodules"
git submodule deinit --all
& ./setup.ps1
git add -A
git commit --no-verify -m "Initialize template using blakeNaccarato/copier-python"
