from collections.abc import Callable
from typing import Any

from django.core.handlers.wsgi import WSGIRequest
from django.http.request import HttpRequest
from django.utils.functional import SimpleLazyObject

def csrf(request: HttpRequest) -> dict[str, SimpleLazyObject]: ...
def debug(request: HttpRequest) -> dict[str, Callable[..., Any] | bool]: ...
def i18n(
    request: WSGIRequest,
) -> dict[str, list[tuple[str, str]] | bool | str]: ...
def tz(request: HttpRequest) -> dict[str, str]: ...
def static(request: HttpRequest) -> dict[str, str]: ...
def media(request: Any) -> Any: ...
def request(request: HttpRequest) -> dict[str, HttpRequest]: ...
