from collections import OrderedDict
from collections.abc import Iterator
from typing import Any

from django.core.files.storage import FileSystemStorage

def matches_patterns(
    path: str, patterns: list[str] | tuple[str] | OrderedDict[Any, Any] = ...
) -> bool: ...
def get_files(
    storage: FileSystemStorage, ignore_patterns: list[str] = ..., location: str = ...
) -> Iterator[str]: ...
def check_settings(base_url: str | None = ...) -> None: ...
