from typing import Any

from django.template.base import Origin
from django.template.engine import Engine

from .base import Loader as BaseLoader

class Loader(BaseLoader):
    dirs: list[str] | None = ...
    def __init__(self, engine: Engine, dirs: list[str] | None = ...) -> None: ...
    def get_dirs(self) -> list[bytes] | list[str]: ...
    def get_contents(self, origin: Origin) -> Any: ...
