from re import Pattern

GET_ITERATOR_CHUNK_SIZE: int = ...

MULTI: str = ...
SINGLE: str = ...
CURSOR: str = ...
NO_RESULTS: str = ...

ORDER_PATTERN: Pattern[str] = ...
ORDER_DIR: dict[str, tuple[str, str]] = ...

INNER: str = ...
LOUTER: str = ...
