from os.path import abspath
from pathlib import Path
from typing import Any

abspathu = abspath

def upath(path: Any) -> Any: ...
def npath(path: Any) -> Any: ...
def safe_join(base: bytes | str, *paths: Any) -> str: ...
def symlinks_supported() -> Any: ...
def to_path(value: Path | str) -> Path: ...
