from typing import Any

from django.apps.config import AppConfig
from django.db.models.base import Model

def popen_wrapper(
    args: list[str], stdout_encoding: str = ...
) -> tuple[str, str, int]: ...
def handle_extensions(extensions: list[str]) -> set[str]: ...
def find_command(
    cmd: str, path: str | None = ..., pathext: str | None = ...
) -> str | None: ...
def get_random_secret_key() -> Any: ...
def parse_apps_and_model_labels(
    labels: list[str],
) -> tuple[set[type[Model]], set[AppConfig]]: ...
def get_command_line_option(argv: Any, option: Any) -> Any: ...
